<?php

namespace App\Http\Controllers;

use App\Models\Countries;
use App\Models\Order;
use App\Models\Product;
use Illuminate\Http\Request;

class CheckoutController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    public function checkout(Product $product, Request $r)
    {

        $r->validate([
            'amount' => 'required|numeric'
        ]);

        // check owner
        if (auth()->id() == $product->user_id) {
            alert()->info(__('You cannot buy your own products.'));
            return back();
        }

        // check stock
        if ($product->stock < 1 && !is_null($product->stock)) {
            abort(404);
        }

        // check min price <= requested amount
        if ($product->price > $r->amount) {
            alert()->error(__(
                'Your offer of :offerAmount is lower than min product price of :productPrice',
                [
                    'offerAmount' => opt('payment-settings.currency_symbol') . $r->amount,
                    'productPrice' => opt('payment-settings.currency_symbol') . $product->price,
                ]
            ));
            return back();
        }

        // send the amount to the checkout review
        $amount = $r->amount;

        // compute shippable countries
        $shippableCountries = collect(Countries::listOf());


        // get user disabled countries
        $userDisabledCountries = user_meta('shipping_country_disabled', true, $product->user_id);
        $userDisabledCountries = json_decode($userDisabledCountries);

        if ($userDisabledCountries) {
            $userDisabledCountries = collect($userDisabledCountries);
            $shippableCountries = $shippableCountries->diff($userDisabledCountries);
        }

        // get shipping cost
        $shippingCost = __('Select Country to Calculate');

        // amount is good, let's show the checkout form
        return view('checkout.review', compact('product', 'amount', 'shippableCountries', 'shippingCost'));
    }

    public function getShippingPrice(Request $r)
    {

        $r->validate(['user' => 'required', 'country' => 'required']);

        $price = user_meta('shipping_country_price', true, $r->user);
        $price = json_decode($price);
        $price = (array) $price;

        if (!$price) {
            $price = user_meta('default_shipping_price', true, $r->user);
        } else {
            if (isset($price[$r->country])) {
                $price = $price[$r->country];
            } else {
                $price = user_meta('default_shipping_price', true, $r->user);
            }
        }

        return response()->json(['price' => $price]);
    }

    public function processOrder(Request $r)
    {

        // get product
        $product = Product::findOrFail($r->product);


        $r->validate([
            'gateway' => 'required',
            'fullname' => 'required_if:products:is_digital,!=,yes',
            'country' => 'required_if:products:is_digital,!=,yes',
            'city' => 'required_if:products:is_digital,!=,yes',
            'state' => 'required_if:products:is_digital,!=,yes',
            'postcode' => 'required_if:products:is_digital,!=,yes',
            'address' => 'required_if:products:is_digital,!=,yes',
            'price' => 'required'
        ]);


        // check stock
        if ($product->stock < 1 && !is_null($product->stock)) {
            abort(403);
        }

        // check min price
        if ($product->price > $r->price) {
            abort(403, __('Invalid price'));
        }

        // check shipping cost for country if the item is not digital
        $shippingCost = 0;

        if ($product->is_digital != 'yes') {
            $shippingCost = user_meta('shipping_country_price', true, $product->user_id);
            $shippingCost = json_decode($shippingCost);
            $shippingCost = (array) $shippingCost;
        }


        if ($product->is_digital != 'yes') {
            if ($shippingCost === false) {
                dd('false');
                $shippingCost = user_meta('default_shipping_price', true, $product->user_id);
            } else {

                if (isset($shippingCost[$r->country])) {
                    $shippingCost = $shippingCost[$r->country];
                } else {
                    $shippingCost = user_meta('default_shipping_price', true, $product->user_id);
                }
            }
        }

        if ($shippingCost === false || is_null($shippingCost)) {
            alert()->error(__('v103.noShippingPrice'));
            return back()->withInput();
        }


        // compute shipping details
        $shippingDetails = [];

        if ($product->is_digital != 'yes') {
            $shippingDetails['fullname'] = $r->fullname;
            $shippingDetails['city'] = $r->city;
            $shippingDetails['state'] = $r->state;
            $shippingDetails['postcode'] = $r->postcode;
            $shippingDetails['country'] = $r->country;
            $shippingDetails['address'] = $r->address;
            $shippingDetails['phone'] = $r->phone;
        }

        // setup order contents
        $orderContents = [];
        $orderContents['product_name'] = $product->product_name;
        $orderContents['username'] = $product->user->profile->username;
        $orderContents['handle'] = $product->user->profile->handle;
        $orderContents['image'] = $product->mainImage();

        // record new order
        $order = new Order();
        $order->buyer_id = auth()->id();
        $order->seller_id = $product->user_id;
        $order->product_id = $r->product;
        $order->shipping_price = $shippingCost;
        $order->product_price = $r->price;
        $order->order_total = $shippingCost + $r->price;
        $order->shipping_details = $shippingDetails;
        $order->gateway = 'PayPal';
        $order->order_contents = $orderContents;
        $order->order_status = 'Pending';
        $order->save();

        return view('checkout.paypal', compact('order'));
    }
}
