<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\ProductImage;
use Intervention\Image\Facades\Image;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;

class FileUploader extends Controller
{
    // auth middleware
    public function __construct()
    {
        $this->middleware("auth");
    }

    // process file upload
    public function process(Request $r)
    {
        // validate image
        $validator = Validator::make(
            $r->all(),
            [
                "files.*" =>
                "required|image|mimes:jpeg,png,jpg|dimensions:min_width=300,min_height=300",
            ]
        );

        if ($validator->fails()) {
            return response()->json(
                [
                    "status" => "error",
                    "message" => $validator->errors()->first(),
                ],
                500
            );
        }

        foreach ($r->file("files") as $file) {
            // compute a file name
            $imageExt = $file->getClientOriginalExtension();
            $fileName = uniqid() . "." . $imageExt;

            // resize
            $imageUpload = Image::make($file);

            // resize for listing (930px witdh auto height)
            $imageUpload
                ->resize(930, null, function ($constraint) {
                    $constraint->aspectRatio();
                    $constraint->upsize();
                })
                ->encode(null, 100)
                ->orientate();

            // store the resized image
            Storage::disk(env("DEFAULT_STORAGE", "public"))->put(
                "product-pics/" . $fileName,
                $imageUpload,
                "public"
            );

            $resp = Storage::disk(env('DEFAULT_STORAGE'))->url("product-pics/" . $fileName);

            return response($resp)->header(
                "Content-Type",
                "text/plain"
            );
        }
    }

    // remove file upload
    public function remove()
    {
        $pic = file_get_contents("php://input");
        $pic = !is_numeric($pic) ? strstr($pic, 'product-pics/') : $pic;


        if (empty(trim($pic))) {
            abort(404);
        }

        // set path
        $path = Storage::disk(env("DEFAULT_STORAGE", "public"))->path("");
        $removePic = $path . $pic;

        // if this pic is already attached to a service, check ownership to allow removal
        $si = ProductImage::where("image", $pic)
            ->orWhere("id", $pic)
            ->first();

        if ($si) {
            // check count
            if (
                ProductImage::where("product_id", $si->product_id)->count() <=
                1
            ) {
                return response()->json(
                    [
                        "status" => "error",
                        "message" => __(
                            "You cannot remove the last image for this product."
                        ),
                    ],
                    500
                );
            } else {
                if ($si->user_id == auth()->user()->id || auth()->user()->isAdmin()) {
                    unlink($path . $si->image);
                    $si->delete();
                } else {
                    abort(403);
                }
            }
        } else {
            // if not attached, it's a temporary image uploaded by the user, so just delete it
            unlink($removePic);
        }
    }

    // load file upload
    public function load(ProductImage $serviceImages)
    {
        $path = Storage::disk(env("DEFAULT_STORAGE", "public"))->get($serviceImages->image);
        $type = $serviceImages->image;
        $ext = explode('.', $type);

        return response($path, 200)
            ->header('Content-type', 'image/' . end($ext))
            ->header('Content-Disposition', 'inline');
    }
}
