<?php

namespace App\Models;

use Carbon\Carbon;
use Overtrue\LaravelFollow\Traits\Followable;
use Overtrue\LaravelFollow\Traits\Follower;
use Overtrue\LaravelLike\Traits\Liker;
use Overtrue\LaravelLike\Traits\Likeable;
use Illuminate\Notifications\Notifiable;
use BeyondCode\Comments\Contracts\Commentator;
use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Foundation\Auth\User as Authenticatable;

class User extends Authenticatable implements Commentator, MustVerifyEmail
{
    use Notifiable;
    use Followable;
    use Follower;
    use Liker;
    use Likeable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
      'name', 'email', 'password',
    ];

    public $casts = ['subscription_expires' => 'datetime'];

    protected $with = ['likes', 'profile'];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
      'password', 'remember_token',
    ];

    // user meta
    public function user_meta()
    {
        return $this->hasMany(UserMeta::class);
    }

    // auto approve comments
    public function needsCommentApproval($model): bool
    {
        return false;
    }

    // relationship to profile
    public function profile()
    {
        return $this->hasOne(Profile::class);
    }

    // get only first name
    public function getFirstNameAttribute()
    {
        $name = $this->name;
        $name = explode(' ', $this->name);

        if (isset($name[0])) {
            return $name[0];
        }

        return '';
    }

    // get only last name
    public function getLastNameAttribute()
    {
        $name = $this->name;
        $name = explode(' ', $this->name);

        if (isset($name[1])) {
            return $name[1];
        }

        return '';
    }

    // is online
    public function isOnline()
    {
        $twoMinutesAgo = Carbon::now()->subMinutes(2);
        return $this->updated_at >= $twoMinutesAgo;
    }

    public function platinumSubscription()
    {
        return $this->hasMany(SaasSubscription::class);
    }

    // is platinum
    public function scopeIsPlatinum()
    {
        return $this->subscription_expires >= now();
    }

    // products limit
    public function scopeCanAddProducts(): bool
    {
        return $this->remainingProducts() === -1 || $this->remainingProducts() > 0;
    }
    public function scopeRemainingProducts(): int
    {
        if ($this->isPlatinum()) {
            return (int) -1;
        }

        $productsLimit = opt('free_shop_limit');

        if ($count = user_meta('lifetimeProducts', true, $this->id)) {
            return (int) $productsLimit - $count;
        }

        return (int) $productsLimit;
    }

    // posts limit
    public function scopeCanAddPosts(): bool
    {
        return $this->remainingPosts() === -1 || $this->remainingPosts() > 0;
    }
    public function scopeRemainingPosts(): int
    {
        if ($this->isPlatinum()) {
            return (int) -1;
        }

        $postsLimit = opt('free_posts_limit');

        if ($count = user_meta('lifetimePosts', true, $this->id)) {
            return (int) $postsLimit - $count;
        }

        return (int) $postsLimit;
    }

    // can receive tips
    public function canReceiveTips(): bool
    {
        if ($this->isPlatinum()) {
            return true;
        }

        return $this->tipsReceived()->count() <= opt('free_tips_limit');
    }

    // can take more paid memberships
    public function canTakePaidMembership(): bool
    {
        if ($this->isPlatinum()) {
            return true;
        }

        return $this->subscribers()->count() <= opt('free_subscribers_limit');
    }

    // has subscription to someone
    public function hasSubscriptionTo(User $creator)
    {
        return $this->subscriptions()->where('subscription_expires', '>=', now())
          ->where('creator_id', $creator->id)
          ->exists();
    }

    // user feed based on who he/she/it follows
    public function feed($lastId = null)
    {
        // get user ids of those who this user is following
        $following = $this->followings->pluck('followable_id')->toArray();

        // also append own posts
        $following[] = $this->id;
        $following = array_unique($following);

        // profile fields
        $profileFields = [
          'id', 'name', 'user_id', 'username', 'creating', 'profilePic',
          'coverPic', 'isVerified', 'isFeatured',
          'monthlyFee', 'minTip', 'yearlyFee'
        ];

        // get posts
        $posts = Post::whereIn('user_id', $following)
          ->with(['profile' => function ($q) use ($profileFields) {
              $q->select($profileFields);
          }, 'postmedia'])
          ->orderByDesc('id')
          ->withCount('likers')
          ->withCount('comments')
          ->take(opt('feedPerPage', 10));

        if (!is_null($lastId)) {
            $posts->where('id', '<', $lastId);
        }

        $posts = $posts->get();

        return $posts;
    }

    // can take payments
    public function canTakePayments()
    {
        return $this->profile->isVerified == 'Yes' && $this->profile->monthlyFee;
    }

    // get this user posts
    public function posts()
    {
        return $this->hasMany(Post::class);
    }

    // received coffees
    public function receivedCoffees()
    {
        return $this->hasMany(Coffee::class, 'creator_id', 'id');
    }

    // given coffees
    public function givenCoffees()
    {
        return $this->hasMany(Coffee::class, 'buyer_id', 'id');
    }

    // incoming messages
    public function incomingMessages()
    {
        return $this->hasMany(Message::class, 'to_id', 'id');
    }

    public function outgoingMessages()
    {
        return $this->hasMany(Message::class, 'from_id', 'id');
    }

    // unread messages (inbox)
    public function unreadInboxCount()
    {
        return $this->incomingMessages()->where('is_read', 'No')->count();
    }

    // withdrawals
    public function withdrawals()
    {
        return $this->hasMany(Withdraw::class);
    }

    // subscriptions
    public function subscriptions()
    {
        return $this->hasMany(Subscription::class, 'subscriber_id', 'id');
    }

    // subscribers
    public function subscribers()
    {
        return $this->hasMany(Subscription::class, 'creator_id', 'id');
    }


    // payment methods
    public function paymentMethods()
    {
        return $this->hasMany(PaymentMethod::class);
    }

    // invoices
    public function invoices()
    {
        return $this->hasMany(Invoice::class);
    }

    // tips given
    public function tipsGiven()
    {
        return $this->hasMany(Tips::class, 'tipper_id', 'id');
    }

    // tips received
    public function tipsReceived()
    {
        return $this->hasMany(Tips::class, 'creator_id', 'id');
    }

    // message unlocks
    public function messageUnlocks()
    {
        return $this->hasMany(Unlock::class, 'tipper_id', 'id');
    }

    // delete listener
    public static function boot()
    {
        parent::boot();

        static::deleting(function ($user) { // before delete() method call this
            // delete posts
            $user->posts()->delete();

            // delete followings / followers
            \DB::delete('DELETE FROM user_follower WHERE following_id = ? OR follower_id = ?', [$user->id, $user->id]);

            // delete likes
            $user->likes()->delete();

            // delete products
            $user->products()->delete();

            // delete comments
            \DB::query('DELETE FROM comments WHERE user_id = ?', $user->id);

            // delete follows
            \DB::query('DELETE FROM followables WHERE user_id = ?', $user->id);
            \DB::query('DELETE FROM followables WHERE followable_id = ?', $user->id);

            // delete notifications
            $user->notifications()->delete();

            // delete messages
            $user->incomingMessages()->delete();
            $user->outgoingMessages()->delete();

            // delete coffees
            $user->receivedCoffees()->delete();
            $user->givenCoffees()->delete();

            \DB::query('DELETE FROM orders WHERE buyer_id = ?', $user->id);
            \DB::query('DELETE FROM orders WHERE seller_id = ?', $user->id);

            // delete creator profile
            $user->profile()->delete();
        });
    }

    public function products()
    {
        return $this->hasMany(Product::class);
    }
}
